==========================================

Feature Based Encoder to Generate Decoding
Energy Saving Bit Streams

==========================================


Additional Flags: 
	-tau 		: Tradeoff parameter between rate and energy optimization (0...1)
	-lambdaE	: Define external lambda for energy (optional)
	-lambdaR	: Define external lambda for rate (optional)
	-rho		: rho parameter for lambda-QP relation (optional)
	-epsilon	: epsilon parameter for lambda-QP relation (optional)

Additional options for config-file: 
	printQP	: Input flag stating if the chosen CTU-based QP is printed (1, else 0). 
	Specific energies for decoding energy model are listed at the end of this document. 

For DERDO, a value for tau must be chosen and a config-gile with specific energies has to be used. Two examples for specific energies are provided: 
- cfg/featureEnergies_FFMPEG.cfg (FFmpeg decoder on a pandaboard (ARMv7), values in Joule)
- cfg/feature_energies_HM_normalized.cfg (HM decoder on a pandaboard (ARMv7), values normalized)


Windows: 
--------
Compilation: 
	Open the HM_vc10.sln file in the "build" folder and compile as release version. 

Execution (Example): 
		 - Open a command prompt (cmd.exe)
		 - Go to the "bin/vc10/Win32/Release" folder
		 - Run "TAppEncoder -i BasketballPass_416x240_50.yuv -c ..\..\..\..\cfg\encoder_randomaccess_main_constantQP.cfg -c ..\..\..\..\cfg\featureEnergies_FFMPEG.cfg -f 16 -fr 50 -b output.bin -wdt 416 -hgt 240 -tau 0.5 -q 32"
		 - An energy saving bit stream is constructed, where tau can be used to determine the tradeoff between rate and energy. 
		 - The estimated decoding energy is printed at the end of the encoding process. 

Linux:
------ 
Compilation: 
	- Open a shell
	- Go to the "build/linux" folder
	- Type "make release" and press return

Execution (Example): 
		 - Open a shell
		 - Go to the "bin" folder
		 - Run "./TAppEncoder -i BasketballPass_416x240_50.yuv -c ../cfg/encoder_randomaccess_main_constantQP.cfg -c ../cfg/featureEnergies_FFMPEG.cfg -f 16 -fr 50 -b output.bin -wdt 416 -hgt 240 -tau 0.5 -q 32"
		 - An energy saving bit stream is constructed, where tau can be used to determine the tradeoff between rate and energy. 
		 - The estimated decoding energy is printed at the end of the encoding process. 



The code related to DERDO is marked with the preprocessor flag "DERDO". 


Citations: 
	- Features for the energy model are given in Herglotz, C. and Springer, D. and Reichenbach, M. and Stabernack, B. and Kaup, A.,
Modeling the Energy Consumption of the HEVC Decoding Process.
In: accepted for IEEE Transactions on Circuits and Systems for Video Technology (TCSVT), Aug 2016. 
	- DERDO is described in Herglotz, C. and Heindel, A. and Kaup, A.,
Decoding-Energy-Rate-Distortion Optimization for Video Coding. submitted to IEEE Transactions on Circuits and Systems for Video Technology (TCSVT), Nov 2016. 



Additional options for config-file (specific energies): 
  lambdaEnergy	:	System dependent lambda for energy optimization
  lambdaRate		: 	Force lambda for RDO. Not used when lambda==-1
  tau			:	Rate-Energy Tradeoff parameter (0...1). 0=RDO, 1=DEDO
  rho			:	Rho value for lambda-QP-relation
  epsilon			:	Epsilon value for lambda-QP-relation
  ang0			:	Intra Angular Depth 1
  ang1			:	Intra Angular Depth 2
  ang2			:	Intra Angular Depth 3
  ang3			:	Intra Angular Depth 4
  pla0			:	Intra planar Depth 1
  pla1			:	Intra planar Depth 2
  pla2			:	Intra planar Depth 3
  pla3			:	Intra planar Depth 4
  dc0			:	Intra dc Depth 1
  dc1			:	Intra dc Depth 2
  dc2			:	Intra dc Depth 3
  dc3			:	Intra dc Depth 4
  hvd0			:	Intra horizontal-vertical-diagonal Depth 1
  hvd1			:	Intra horizontal-vertical-diagonal Depth 2
  hvd2			:	Intra horizontal-vertical-diagonal Depth 3
  hvd3			:	Intra horizontal-vertical-diagonal Depth 4
  intraCU			:	Number of intra coded CUs
  Islice			:	Intra slice
  PBslice			:	Inter slice
  E0			:	Offset energy
  noMPM			:	Additional Energy when intra mode is not a most probable mode
  coeff			:	Nonzero residual coefficient
  CSBF			:	Coded Subblock flag
  coeffg1			:	Coefficient greater 1
  val			:	Coefficient values
  TrIntraY0			:	Intra luma transformation depth 1
  TrIntraY1			:	Intra luma transformation depth 2
  TrIntraY2			:	Intra luma transformation depth 3
  TrIntraY3			:	Intra luma transformation depth 4
  TrIntraC0			:	Intra chroma transformation depth 1
  TrIntraC1			:	Intra chroma transformation depth 2
  TrIntraC2			:	Intra chroma transformation depth 3
  TrIntraC3			:	Intra chroma transformation depth 4
  TrInterY0			:	inter luma transformation depth 1
  TrInterY1			:	inter luma transformation depth 2
  TrInterY2			:	inter luma transformation depth 3
  TrInterY3			:	inter luma transformation depth 4
  TrInterC0			:	inter chroma transformation depth 1
  TrInterC1			:	inter chroma transformation depth 2
  TrInterC2			:	inter chroma transformation depth 3
  TrInterC3			:	inter chroma transformation depth 4
  TSF			:	Transform skipping
  MVD			:	Motion vector difference
  copypel			:	Energy for a single pel that needs to be copied
  fracpelHor0			:	Fractional pel energy horizontal depth 0
  fracpelHor1			:	Fractional pel energy horizontal depth 1
  fracpelHor2			:	Fractional pel energy horizontal depth 2
  fracpelHor3			:	Fractional pel energy horizontal depth 3
  fracpelVer0			:	Fractional pel energy vertical depth 0
  fracpelVer1			:	Fractional pel energy vertical depth 1
  fracpelVer2			:	Fractional pel energy vertical depth 2
  fracpelVer3			:	Fractional pel energy vertical depth 3
  chrHalfpel0			:	Chroma halfpel energy depth 0
  chrHalfpel1			:	Chroma halfpel energy depth 1
  chrHalfpel2			:	Chroma halfpel energy depth 2
  chrHalfpel3			:	Chroma halfpel energy depth 3
  bi			:	Bidirectional prediction for 4x4 block
  skip0			:	Skipping depth 0
  skip1			:	Skipping depth 1
  skip2			:	Skipping depth 2
  skip3			:	Skipping depth 3
  merge0			:	Merging 2Nx2N depth 0
  merge1			:	Merging 2Nx2N depth 1
  merge2			:	Merging 2Nx2N depth 2
  merge3			:	Merging 2Nx2N depth 3
  mergeSMP0			:	Merging SMP depth 0
  mergeSMP1			:	Merging SMP depth 1
  mergeSMP2			:	Merging SMP depth 2
  mergeSMP3			:	Merging SMP depth 3
  mergeAMP0			:	Merging AMP depth 0
  mergeAMP1			:	Merging AMP depth 1
  mergeAMP2			:	Merging AMP depth 2
  mergeAMP3			:	Merging AMP depth 3
  inter0			:	Inter 2Nx2N depth 0
  inter1			:	Inter 2Nx2N depth 1
  inter2			:	Inter 2Nx2N depth 2
  inter3			:	Inter 2Nx2N depth 3
  interSMP0			:	Inter SMP depth 0
  interSMP1			:	Inter SMP depth 1
  interSMP2			:	Inter SMP depth 2
  interSMP3			:	Inter SMP depth 3
  interAMP0			:	Inter AMP depth 0
  interAMP1			:	Inter AMP depth 1
  interAMP2			:	Inter AMP depth 2
  interAMP3			:	Inter AMP depth 3
  Bs0			:	Boundary Strength 0
  Bs1			:	Boundary Strength 1
  Bs2			:	Boundary Strength 2
  
  SAO_Y_EO			:	SAO: Edge Offset case 0, luma
  SAO_C_EO			:	SAO: Edge Offset case 0, chroma
  SAO_C_BO			:	SAO: Band Offset, chroma
  SAO_allComps			:	Correcting if all modes are SAO-filtered
  e_transmissionPerBit			:	Average transmission energy (via network) per bit




